@@ -25,7 +25,8 @@ OBJECTS={
     "prices": 3,
     "dividends": 4,
     "statistic": 5,
-    "equity_statistic":6
+    "equity_statistic":6,
+    "api_key": 7
 }
 logger = outputter.Logger("files", settings.LOG_LEVEL)
 
@@ -46,8 +47,8 @@ def generate_timestamp(args):
 def load_file(file_name):
     with open(file_name, 'r') as infile:
         if settings.FILE_EXT == "json":
-            prices = json.load(infile)
-        return prices
+            file = json.load(infile)
+        return file
         # TODO: implement other file loading extensions
 
 def save_file(file_to_save, file_name):
@@ -56,8 +57,10 @@ def save_file(file_to_save, file_name):
             try:
                 json.dump(file_to_save, outfile)
                 return True
-            except:
+            except Exception as e:
+                logger.info(f'A {e.__class__} exception occured.')
                 return False
+            
         # TODO: implement other file saving extensions.
 
 
@@ -92,6 +95,8 @@ def store_local_object(local_object, value, args):
             args: { 'stat_symbol' : value } \n 
         6. if local_object = equity_statistic \n 
             args: { 'ticker' : value , 'equity_stat_symbol' : value } \n \n
+        7. if local_object = api_key \n
+            args: { 'key_name': value, 'key_value': value }
 
     """
     if settings.LOCAL_CACHE:
@@ -115,6 +120,9 @@ def store_local_object(local_object, value, args):
             file_name = os.path.join(settings.CACHE_DIR, f'{timestamp}_{args["stat_symbol"]}_{settings.CACHE_STAT_KEY}.{settings.FILE_EXT}')
         if local_object == OBJECTS['equity_statistic']:
             file_name = os.path.join(settings.CACHE_DIR,f'{timestamp}_{args["ticker"]}_{args["equity_stat_symbol"]}_{settings.CACHE_EQUITY_KEY}.{settings.FILE_EXT}')
+        if local_object == OBJECTS['api_key']:
+            file_name = os.path.join(settings.COMMON_DIR, f'{args["key_name"]}.{settings.FILE_EXT}')
+        
         buffer_store= os.path.join(settings.CACHE_DIR, file_name)
         return save_file(file_to_save=value, file_name=buffer_store)
     return False
@@ -174,8 +182,12 @@ def retrieve_local_object(local_object, args):
             file_name = os.path.join(settings.CACHE_DIR,f'{timestamp}_{args["ticker"]}_{args["equity_stat_symbol"]}_{settings.CACHE_EQUITY_KEY}.{settings.FILE_EXT}')
             logger.debug(f'Checking for {args["ticker"]}\'s {args["equity_stat_symbol"]} statistics in local cache')
         
+        elif local_object == OBJECTS['api_key']:
+            file_name = os.path.join(settings.COMMON_DIR, f'{args["key_name"]}.{settings.FILE_EXT}')
+            logger.debug(f'Checking for {args["key_name"]} API key in local commons')
+
         if file_name is not None and os.path.isfile(file_name):
-            logger.debug(f'Loading in local cache')
+            logger.debug('Loading in local cache')
             results = load_file(file_name = file_name)
             return results
 
@@ -379,26 +391,6 @@ def get_asset_type(symbol):
     # default to equity for overlap until a better method is determined. 
     return settings.ASSET_EQUITY
 
-def get_trading_period(asset_type):
-    """
-    Description
-    -----------
-    Returns the value of one trading day measured in years of the asset_type passed in as an argument.
-
-    Parameters
-    ----------
-    1. asset_type : str\n
-    
-    A string that represents a type of tradeable asset. Types are statically accessible through the ` settings` variables: ASSET_EQUITY and ASSET_CRYPTO.
-    """
-    if asset_type is None:
-        return False
-    if asset_type == settings.ASSET_CRYPTO:
-        return (1/365)
-    if asset_type == settings.ASSET_EQUITY:
-        return settings.ONE_TRADING_DAY
-    return settings.ONE_TRADING_DAY
-
 def get_watchlist():
     """
     Description
@@ -444,8 +436,16 @@ def add_watchlist(new_tickers):
             json.dump(current_tickers, outfile)
         # TODO: implement other file extensions
 
+def format_profiles(profiles):
+    profiles_format = []
+    for key, value in profiles.items():
+        holding = value
+        holding['ticker'] = key
+        profiles_format.append(holding)
+    return profiles_format
+
 def format_allocation(allocation, portfolio, investment=None):
-    allocation_format = {}
+    allocation_format = []
 
     if investment is not None:
         shares = portfolio.calculate_approximate_shares(x=allocation, total=investment)
@@ -454,14 +454,15 @@ def format_allocation(allocation, portfolio, investment=None):
     annual_volatility = portfolio.volatility_function(x=allocation) 
     annual_return = portfolio.return_function(x=allocation)
 
-    for j in range(len(portfolio.tickers)):
-        allocation_format[j] = {}
-        allocation_format[j]['ticker'] = portfolio.tickers[j]
-        allocation_format[j]['allocation'] = round(allocation[j], settings.ACCURACY)
+    for j, item in enumerate(portfolio.tickers):
+        holding = {}
+        holding['ticker'] = item
+        holding['allocation'] = round(allocation[j], settings.ACCURACY)
         if investment is not None:
-            allocation_format[j]['shares'] = float(shares[j])
-        allocation_format[j]['annual_return'] = round(portfolio.mean_return[j], settings.ACCURACY) 
-        allocation_format[j]['annual_volatility'] = round(portfolio.sample_vol[j], settings.ACCURACY)
+            holding['shares'] = float(shares[j])
+        holding['annual_return'] = round(portfolio.mean_return[j], settings.ACCURACY) 
+        holding['annual_volatility'] = round(portfolio.sample_vol[j], settings.ACCURACY)
+        allocation_format.append(holding)
 
     json_format = {}
     json_format['holdings'] = allocation_format
@@ -476,8 +477,8 @@ def format_allocation(allocation, portfolio, investment=None):
 
 def format_frontier(portfolio, frontier, investment=None):
     json_format = {}
-    for i in range(len(frontier)):
-        json_format[f'portfolio_{i}'] = format_allocation(allocation=frontier[i], portfolio=portfolio, 
+    for i, item in enumerate(frontier):
+        json_format[f'portfolio_{i}'] = format_allocation(allocation=item, portfolio=portfolio, 
                                                             investment=investment)
 
     return json_format
@@ -486,8 +487,8 @@ def format_moving_averages(tickers, averages_output):
     these_moving_averages, dates = averages_output
 
     response = {}
-    for i in range(len(tickers)):
-        ticker_str=f'{tickers[i]}'
+    for i, item in enumerate(tickers):
+        ticker_str=f'{item}'
         MA_1_str, MA_2_str, MA_3_str = f'{ticker_str}_MA_1', f'{ticker_str}_MA_2', f'{ticker_str}_MA_3'    
 
         subresponse = {}
@@ -499,8 +500,8 @@ def format_moving_averages(tickers, averages_output):
         else:
             subsubresponse_1, subsubresponse_2, subsubresponse_3 = {}, {}, {}
     
-            for j in range(len(dates)):
-                date_str=helper.date_to_string(dates[j])
+            for j, this_item in enumerate(dates):
+                date_str=helper.date_to_string(this_item)
                 subsubresponse_1[date_str] = these_moving_averages[i][0][j]
                 subsubresponse_2[date_str] = these_moving_averages[i][1][j]
                 subsubresponse_3[date_str] = these_moving_averages[i][2][j]
@@ -515,12 +516,16 @@ def format_moving_averages(tickers, averages_output):
 
 def format_correlation_matrix(tickers, correlation_matrix):
     response = {}
-    for i in range(len(tickers)):
+    for i, item in enumerate(tickers):
         # correlation_matrix[i][i]
         for j in range(i+1, len(tickers)):
-            response[f'{tickers[i]}_{tickers[j]}_correlation'] = correlation_matrix[j][i]
+            response[f'{item}_{tickers[j]}_correlation'] = correlation_matrix[j][i]
     return response
     
+def save_profiles(profile, file_name):
+    save_format = format_profiles(profile)
+    save_file(file_name==save_format, file_name=file_name)
+
 def save_allocation(allocation, portfolio, file_name, investment=None):
     save_format = format_allocation(allocation=allocation, portfolio=portfolio, investment=investment)
     save_file(file_to_save=save_format, file_name=file_name)
