@@ -46,12 +46,13 @@ def validate_function_usage(selection, args, wrapper_function, required_length=1
         elif(exact and (len(args)==required_length)):
             wrapper_function()
         else:
-            logger.comment('Error encountered while calculating. Try -ex flag for example usage.')
+            logger.comment(f'Invalid number of arguments for \'{selection}\' function.')
 
 def do_program():
     if len(sys.argv)>0:
         logger.debug('Parsing and invoking command line arguments')
         opt = sys.argv[1]
+        logger.debug(f'Selected Function: {opt}')
         
         # single argument functions
         ### FUNCTION: Help Message
@@ -100,15 +101,30 @@ def do_program():
         ### FUNCTION: Risk Free Rate
         elif opt == formatter.FUNC_ARG_DICT['risk_free_rate']:
             outputter.title_line("Risk Free Rate")
-            outputter.scalar_result(calculation=formatter.RISK_FREE_TITLE, 
+            outputter.scalar_result(calculation=formatter.RISK_FREE_TITLE.format(settings.RISK_FREE_RATE), 
                                     result=services.get_risk_free_rate(), 
                                     currency=False)
 
+        ### FUNCTION: Store Key
+            # NOTE: not technically a single argument function, but requires special parsing, so
+            #       treat it here.
+        elif opt == formatter.FUNC_ARG_DICT['store']:
+            keyvalue = sys.argv[2].split("=")
+            if keyvalue[0] in ["ALPHA_VANTAGE_KEY", "QUANDL_KEY", "IEX_KEY"]:
+               keystore = {}
+               keystore[keyvalue[0]] = keyvalue[1]
+               args = { 'key_name': keyvalue[0], 'key_value': keyvalue[1]}
+               files.store_local_object(local_object=files.OBJECTS['api_key'],value=keystore, args=args)
+            else:
+                outputter.comment(f'Key of {keyvalue[0]} not recognized. See -help for more information.')
+            pass
+
         ### FUNCTION: Purge Data Directories
         elif opt == formatter.FUNC_ARG_DICT["purge"]:
-            logger.comment(f'Clearing {settings.STATIC_DIR} and {settings.CACHE_DIR}')
+            logger.comment(f'Clearing {settings.STATIC_DIR}, {settings.CACHE_DIR} and {settings.CACHE_DIR}')
             files.clear_directory(directory=settings.STATIC_DIR, retain=True)
             files.clear_directory(directory=settings.CACHE_DIR, retain=True)
+            files.clear_directory(directory=settings.COMMON_DIR, retain=True)
 
         ### FUNCTION: Yield Curve
         elif opt == formatter.FUNC_ARG_DICT['yield_curve']:
@@ -125,7 +141,7 @@ def do_program():
             xtra_args, xtra_values, main_args = helper.separate_and_parse_args(args)
             xtra_list = helper.format_xtra_args_list(xtra_args, xtra_values)
             logger.log_arguments(main_args=main_args, xtra_args=xtra_args, xtra_values=xtra_values)
-            exact = False
+            exact, selected_function = False, None
 
             outputter.title_line('Results')
             outputter.print_line()
@@ -134,7 +150,7 @@ def do_program():
             if opt == formatter.FUNC_ARG_DICT['asset_type']:
                 def cli_asset_type():
                     for arg in main_args:
-                        asset_type = markets.get_asset_type(arg)
+                        asset_type = files.get_asset_type(arg)
                         if asset_type:
                             outputter.string_result(f'asset_type({arg})', asset_type)
                         else: 
@@ -253,8 +269,8 @@ def do_program():
                 selected_function, required_length = cli_moving_averages, 1
 
             ### FUNCTION: Optimize Portfolio Variance/Volatility
-            elif opt == formatter.FUNC_ARG_DICT['optimize_portfolio']:
-                def cli_optimize_portfolio():
+            elif opt == formatter.FUNC_ARG_DICT['optimize_portfolio_variance']:
+                def cli_optimize_portfolio_variance():
                     portfolio = Portfolio(tickers=main_args, start_date=xtra_list['start_date'], end_date=xtra_list['end_date'])
                     if xtra_list['optimize_sharpe']:
                         allocation = optimizer.maximize_sharpe_ratio(portfolio=portfolio, target_return=xtra_list['target'])
@@ -265,8 +281,21 @@ def do_program():
                     if xtra_list['save_file'] is not None:
                         files.save_allocation(allocation=allocation, portfolio=portfolio, file_name=xtra_list['save_file'],
                                                 investment=xtra_list['investment'])
-                selected_function, required_length = cli_optimize_portfolio, 2
+                selected_function, required_length = cli_optimize_portfolio_variance, 2
             
+            elif opt == formatter.FUNC_ARG_DICT['optimize_portfolio_conditional_var']:
+                def cli_optimize_conditional_value_at_risk():
+                    portfolio = Portfolio(tickers=main_args, start_date=xtra_list['start_date'], end_date=xtra_list['end_date'])
+                    allocation = optimizer.optimize_conditional_value_at_risk(portfolio=portfolio,
+                                                                                prob=xtra_list['probability'],
+                                                                                expiry=xtra_list['expiry'],
+                                                                                target_return=xtra_list['target'])
+                    outputter.optimal_result(portfolio=portfolio, allocation=allocation, investment=xtra_list['investment'])
+
+                    if xtra_list['save_file'] is not None:
+                        files.save_allocation(allocation=allocation, portfolio=portfolio, file_name=xtra_list['save_file'],
+                                                investment=xtra_list['investment'])
+                selected_function, required_length = cli_optimize_conditional_value_at_risk, 2
             ### FUNCTION: Plot Dividend History With Linear Regression Model
             elif opt == formatter.FUNC_ARG_DICT['plot_dividends']:
                 def cli_plot_dividends():
@@ -319,7 +348,7 @@ def do_program():
                     for arg in main_args:
                         prices = services.get_daily_price_history(ticker=arg, start_date=xtra_list['start_date'],
                                                                     end_date=xtra_list['end_date'])
-                        asset_type = markets.get_asset_type(symbol=arg)
+                        asset_type = files.get_asset_type(symbol=arg)
                         all_prices[arg] = {}
                         for date in prices:
                             price = services.parse_price_from_date(prices=prices, date=date, asset_type=asset_type)
@@ -331,24 +360,29 @@ def do_program():
                 selected_function, required_length = cli_price_history, 1
 
             ### FUNCTION: Risk-Return Profile
-            elif opt == formatter.FUNC_ARG_DICT["risk_return"]:
+            elif opt == formatter.FUNC_ARG_DICT["risk_profile"]:
                 def cli_risk_return():
                     profiles = {}
-                    failed = False
                     for arg in main_args:
-                        result = statistics.calculate_risk_return(ticker=arg, start_date=xtra_list['start_date'], 
-                                                                    end_date=xtra_list['end_date'])
-                        if result:
-                            outputter.scalar_result(calculation=f'mean_{arg}', result=result['annual_return'],
-                                                    currency=False)
-                            outputter.scalar_result(calculation=f'vol_{arg}', result=result['annual_volatility'],
-                                                    currency=False)
-                            profiles[arg] = result
-                        else:
-                            failed = True
-                            logger.comment('Error Encountered While Calculating. Try -ex Flag For Example Usage.')
-                        if not failed and xtra_list['save_file'] is not None:
-                            files.save_file(file_to_save=profiles, file_name=xtra_list['save_file'])
+                        try:
+                            profiles[arg] = statistics.calculate_risk_return(ticker=arg, start_date=xtra_list['start_date'], 
+                                                                                end_date=xtra_list['end_date'])
+                            profiles[arg]['sharpe_ratio'] = markets.sharpe_ratio(ticker=arg, start_date=xtra_list['start_date'],
+                                                                                end_date=xtra_list['end_date'])
+                            profiles[arg]['asset_beta'] = markets.market_beta(ticker=arg, start_date=xtra_list['start_date'],
+                                                                                end_date=xtra_list['end_date'])
+                            profiles[arg]['equity_cost'] = markets.cost_of_equity(ticker=arg, start_date=xtra_list['start_date'],
+                                                                                end_date=xtra_list['end_date'])
+
+                            if xtra_list['save_file'] is not None:
+                                files.save_profiles(file_to_save=profiles, file_name=xtra_list['save_file'])
+
+                        except statistics.PriceError as pe:
+                            logger.comment(str(pe))
+                        except statistics.SampleSizeError as se:
+                            logger.comment(str(se))
+                    outputter.risk_profile(profiles=profiles)
+
                 selected_function, required_length = cli_risk_return, 1
 
             ### FUNCTION: Model Discount Screener 
@@ -400,9 +434,10 @@ def do_program():
                 logger.comment('No function supplied. Please review Function Summary below and re-execute with appropriate arguments.')
                 outputter.help_msg()
             
-            validate_function_usage(selection=opt, args=main_args, 
-                                    wrapper_function=selected_function, 
-                                    required_length=required_length, exact=exact)
+            if selected_function is not None:
+                validate_function_usage(selection=opt, args=main_args, 
+                                        wrapper_function=selected_function, 
+                                        required_length=required_length, exact=exact)
             outputter.print_line()
     else:
         logger.comment('No arguments Supplied. Please review function summary below and re-execute with appropriate arguments.')
